@@ -6,9 +6,9 @@
     use DBI qw(:sql_types);
 
     @EXPORT = qw(); # Do NOT @EXPORT anything.
-    $VERSION = sprintf("%d.%02d", q$Revision: 11.5 $ =~ /(\d+)\.(\d+)/o);
+    $VERSION = sprintf("%d.%02d", q$Revision: 11.9 $ =~ /(\d+)\.(\d+)/o);
 
-#   $Id: ExampleP.pm,v 11.5 2002/06/14 13:11:26 timbo Exp $
+#   $Id: ExampleP.pm,v 11.9 2003/02/28 17:50:06 timbo Exp $
 #
 #   Copyright (c) 1994,1997,1998 Tim Bunce
 #
@@ -26,6 +26,10 @@
 	SQL_INTEGER, SQL_INTEGER, SQL_INTEGER,
 	SQL_INTEGER, SQL_INTEGER, SQL_VARCHAR);
     @stattypes{@statnames} = @stattypes;
+    @statprec = ((10) x (@statnames-1), 1024);
+    @statprec{@statnames} = @statprec;
+    die unless @statnames == @stattypes;
+    die unless @statprec  == @stattypes;
 
     $drh = undef;	# holds driver handle once initialised
     $err = 0;		# The $DBI::err value
@@ -39,7 +43,7 @@
 	    'Name' => 'ExampleP',
 	    'Version' => $VERSION,
 	    'Attribution' => 'DBD Example Perl stub by Tim Bunce',
-	    }, ['example implementors private data']);
+	    }, ['example implementors private data '.__PACKAGE__]);
 	$drh;
     }
 
@@ -68,9 +72,6 @@
 	return ("dbi:ExampleP:dir=.");	# possibly usefully meaningless
     }
 
-    sub disconnect_all {
-	# we don't need to tidy up anything
-    }
     sub DESTROY { undef }
 }
 
@@ -89,7 +90,7 @@
 
 	my ($outer, $inner) = DBI::_new_sth($dbh, {
 	    'Statement'     => $statement,
-	}, ['example implementors private data']);
+	}, ['example implementors private data '.__PACKAGE__]);
 
 	my @fields = ($fields eq '*')
 			? keys %DBD::ExampleP::statnames
@@ -108,9 +109,7 @@
 	$outer->STORE('NULLABLE' => [ (0) x @fields ]);
 	$outer->STORE('NUM_OF_FIELDS' => scalar(@fields));
 	$outer->STORE('NUM_OF_PARAMS' => ($dir !~ /\?/) ? 0 : 1);
-	# should do better here:
-	$outer->STORE('SCALE'     => undef);
-	$outer->STORE('PRECISION' => undef);
+	$outer->STORE('SCALE'     => [ (0) x @fields ] );
 
 	$outer;
     }
@@ -326,6 +325,8 @@
 		$sth->finish;
 		return;
 	    }
+	    # untaint $f so that we can use this for DBI taint tests
+	    ($f) = ($f =~ m/^(.*)$/);
 	    my $file = $haveFileSpec
 		? File::Spec->catfile($dir, $f) : "$dir/$f";
 	    # put in all the data fields
@@ -356,8 +357,10 @@
 	# either return dynamic values that cannot be precomputed
 	# or fetch and cache attribute values too expensive to prefetch.
 	if ($attrib eq 'TYPE'){
-	    my @t = @DBD::ExampleP::stattypes{ @{ $sth->{NAME} } };
-	    return \@t;
+	    return [ @DBD::ExampleP::stattypes{ @{ $sth->{NAME_lc} } } ];
+	}
+	elsif ($attrib eq 'PRECISION'){
+	    return [ @DBD::ExampleP::statprec{  @{ $sth->{NAME_lc} } } ];
 	}
 	elsif ($attrib eq 'ParamValues') {
 	    my $dbd_param = $sth->{dbd_param} || [];
